;DDS Controller Program
; Simple version for control of AM band frequencies
; with two rotary hex switches as input
;
; Now uses direct calculation of FTW instead of using lookup tables
; Same program works with either 125 or 180 MHz ref osc by jumper setting
;
; 2023-11-17 Revised to correct control bits for AD9850 Module
;
; **** End of Revision Notes ****
;
;
; Assembler directives
;
; I/O Port Assignments
; 
; A0 - Input - RotSw LSD2
; A1 - Input - RotSw LSD8
; A2 - Input - RotSw MSD2
; A3 - Input* - RotSw MSD4
; A4 - Input - RotSw LSD1
; A5 - Input - RotSw LSD4
; C0 - Input - RotSw MSD8
; C1 - Output - DDSclk
; C2 - Output - DDSfq_ud
; C3 - Output - DDSdat (after startup)
; C3 - input - refosc select (at startup)
; C4 - Input - freq offset flag
; C5 - Input - RotSw MSD1
;

;DDS chip control lines
    #define DDSfq_ud portC,2
    #define DDSclk portC,1
    #define DDSdata portC,3
    #define plusOffset portC,4 ;Add offset frequency flag
    #define ref125 ProgOptions,3

;   Other Misc. Equates
eebase equ 0x2100 ;base address of EERAM

; Delay constants for 4 MHz clock
; 1ms and 10ms are exact, for critical timing
LongDelayMpy equ d'15'
DebounceConst equ d'24'
Delay10msLong equ d'12' ;Delay constants for 4.000 MHz clock
Delay10msShort equ d'248'
Delay1msLong equ d'1'
Delay1msShort equ d'72'
Delay40usShort equ d'12'

    #include p16f630.inc

    __config _INTRC_OSC_NOCLKOUT & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF

; Note that the following macro definitions are boiler plate code
; that I attach to most of my projects, in case they're needed
; They may not all be used, but unused macro definitions don't waste
; program memory. So, they do no harm.

;Macro definitions
bank0 macro
    bcf STATUS,RP0
    endm
bank1 macro
    bsf STATUS,RP0
    endm
;More intuitive skip definitions
skpeq macro
    btfss status,z
    endm
skpneq macro
    btfsc status,z
    endm
skpneg macro
    btfsc status,c
    endm
skppos macro
    btfss status,c
    endm
dde macro arg ;Store a 16 bit value in adjacent memory locs
    de low arg
    de high arg
    endm
SaveAccHi macro rName ;Store 4 high bytes of acc to spec'd register set
    movlw rName
    call SaveAccHiInd
    endm
SaveAccLo macro rName ;Store 4 low bytes of acc to spec'd register set
    movlw rName
    call SaveAccLoInd
    endm
LoadAccHi macro rName ;Load 4 high bytes of acc from spec'd register set
    movlw rName
    call LoadAccHiInd
    endm
LoadAccLo macro rName ;Load 4 low bytes of acc from spec'd register set
    movlw rName
    call LoadAccLoInd
    endm
LoadRLo macro rName ;Load 4 low bytes of aux reg from spec'd register set
    movlw rName
    call LoadRxLoInd
    endm
LoadRHi macro rName ;Load 4 high bytes of aux reg from spec'd register set
    movlw rName
    call LoadRxHiInd
    endm
ClearNregisters macro nReg, rName ;Clear nReg registers starting at rName
    movlw nReg
    movwf xfrCount
    movlw rName
    call ClearNregistersInd
    endm
ShiftUpNregisters macro nReg, rName ;Shift nReg registers starting at rName
    movlw nReg
    movwf xfrCount
    movlw rName
    call ShiftUpNregistersInd
    endm

;
;variable memory area
    cblock 0x20
    dlyctrL,dlyctrH ;for delay routines
;   DDS variables:
    freqDigits:8 ;8 digit frequency in Hz, F(0) is LSB, F(7) is MSB
    fBase0,fBase1,fBase2,fBase3,fBase4,fBase5 ;FTW base value loaded from eeram
    fqOffset0,fqOffset1,fqOffset2,fqOffset3,fqOffset4,fqOffset5,fqOffset6,fqOffset7
    AdjDigits0,AdjDigits1,AdjDigits2,AdjDigits3,AdjDigits4,AdjDigits5,AdjDigits6,AdjDigits7
    cw0,cw1,cw2,cw3 ;Storage for DDS control word
;   Math Library variables
    acc0,acc1,acc2,acc3,acc4,acc5,acc6,acc7 ;accumulator for 32/64 bit math
    R0,R1,R2,R3,R4,R5,R6,R7 ; 32/64 bit math aux. registers
    count,count1,temp ;misc temporary registers
    xfrCount ;used to count registers moved by the load, save & clear routines
    lastInput,newInput,lastOfstFlag,ProgOptions ; rotary sw. variables
    endc

;Reset and interrupt vectors
RST code 0x0
    goto	Main	;Entry point for program from Reset
    nop
    nop
    nop
    return	;Vector for interrupt routine

;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;
Main ;Start of main program
; PIC Hardware initialization
    clrf status ; clear status reg
    clrf portC ;Do this immediately to prevent problems with DDS reset
    clrf pclath ; clear PCH
    clrf intcon
    bank0 ; Sel Bank 0
    clrf portA
    clrf ProgOptions
    movlw 0x07  ;comparator off
    movwf cmcon ;disable comparator, enable digital portA
    bank1 ; Sel Bank 1
    call 0x3FF ; calls RETLW with factory setting
    movwf OSCCAL ; Set int OSC to factory calibrated
    clrf option_reg
; Temp config of portC to read option settings
    movlw b'00111011'
    movwf TRISC
    bank0
    movf portC,w
    movwf ProgOptions ;bit3 indicates ref osc frequency 1=125, 0=180
    bank1
; Continue with permanent TRIS settings
    movlw b'00110001' ; Config port C for half & half
    movwf TRISC
    movlw b'00111111' ; Config port A all inputs
    movwf TRISA
    movlw b'00111111'
    movwf wpua  ;set portA inputs weak pull-ups
; Interrupt initialization - No interrupts are used in this program.
    bank1
    clrf pie1   ;Disable all peripheral interrupts
    movlw b'00001000'  ;Disable interrupts
    movwf intcon    ;
; Program initialization
    bank0
; DDS initialization
    bcf DDSfq_ud
    bsf DDSclk ; holding clock high for at least 1ms resets DDS
    call Delay10mS
    bcf DDSclk
    call Delay1mS
    call SetDDSserial ;DDS starts up in parallel mode. This sets serial mode
    call SetOffsetValue ;
    call getBaseVal ;Set up base value of FTW for direct calculation
    movlw 0xFF
    movwf newInput ;Set to invalid value to force initial DDS transfer
MainLoop
    call Delay10mS
    call RotSwRead ;read the rotary switch inputs
    iorlw 0 ;test return status 0=no change, 1=new frequency
    skpeq
    goto DoUpdate ; Switch inputs changed, so send new FTW to DDS
    call OffsetFlagRead
    iorlw 0
    skpeq
DoUpdate
    call FreqUpdate ; Offset flag changed, so send new FTW to DDS
    goto MainLoop ;repeat forever.
;
;
;  Project Subroutines
;
;
getBaseVal
    movlw ftw1-eebase ;read FTW base value from eeram and store in registers
    btfsc ref125
    movlw ftw2-eebase
    call getEEram
    movwf fBase0
    call getNxtEEram
    movwf fBase1
    call getNxtEEram
    movwf fBase2
    call getNxtEEram
    movwf fBase3
    call getNxtEEram
    movwf fBase4
    call getNxtEEram
    movwf fBase5
    bank0
    return

SetOffsetValue ;Set the value of the optional frequency offset 1600 kHz
    movlw fOffsetSave-eebase ;read freq offset value from eeram & save
    call getEEram
    movwf fqOffset7
    call getNxtEEram
    movwf fqOffset6
    call getNxtEEram
    movwf fqOffset5
    call getNxtEEram
    movwf fqOffset4
    call getNxtEEram
    movwf fqOffset3
    call getNxtEEram
    movwf fqOffset2
    call getNxtEEram
    movwf fqOffset1
    call getNxtEEram
    movwf fqOffset0
    bank0
    return

OffsetFlagRead
    movf lastOfstFlag,w
    movwf temp
    movf portC,w
    movwf lastOfstFlag
    xorwf temp,f
    btfss temp,4
    retlw 0 ;no change so exit with w=0
    retlw 1 ;status changed, exit with w=1

RotSwRead
    movf newInput,w
    movwf lastInput ;Save previous input so we can tell if there's a change
; RotSw inputs are arranged for convenient wiring & must be unscrambled
    swapf portC,w
    andlw 0x12
    movwf newInput
    movf portA,w
    movwf temp
    andlw 0x0C
    iorwf newInput,f
    bcf status,c
    rrf newInput,f ;newInput now has MSD in low nibble
    swapf newInput,f ;move to high nibble
; The next digit is a bit more complicated
    movlw 0x33
    andwf temp,f ;strip off all but LSD bits
    movf temp,w ; LSD1 is in correct position in high nibble
    andlw 0x10 ; mask off everything else
    rlf temp,f ; LSD2&4 are in correct positions
    iorwf temp,w
    andlw 0x52 ;mask off everything else
    iorlw 0x08 ;set bit position  for MSD8
    rlf temp,f ; LSD8 is now in correct position
    andwf temp,f ;Clear all bits in temp that are not set in w
    andlw 0xF7 ; clear LSD8 in w
    iorwf temp,w
    movwf temp
    swapf temp,w
    iorwf temp,w ;w now contains LSD in both nibbles
    andlw 0x0F ;clear high nibble
    iorwf newInput,f ;newInput now contains both MSD and LSD 
    comf newInput,f ;Reverse sense of inputs
; See if input has changed from previous switch read
    movf newInput,w
    xorwf lastInput,w
    skpneq ;check if input has changed
    retlw 0 ;no change. exit with w=0
SetDigits
    ClearNregisters 8,freqDigits
    movf newInput,w
    movwf freqDigits+4 ;set the 10 kHz digit
; Need to check here to see if the 10 kHz digit is >9 & fix if it is.
    movlw 0x06
    addwf freqDigits+4,w
    btfsc status,dc ;if digit carry is set, value is too big
    clrf freqDigits+4 ;value is invalid so set to zero
    swapf newInput,w
    andlw 0x0F
    movwf temp
    movlw 0x06 ; Decimal adjust values > 9 in MSD
    addwf temp,w
    btfsc status,dc
    movwf temp
    movf temp,w
    movwf freqDigits+5 ;set the 100 kHz digit
    swapf temp,w
    movwf freqDigits+6 ;set the 1000 kHz digit
    movlw 0x0F ; mask the off the high bits of the digit registers
    andwf freqDigits+4,f
    andwf freqDigits+5,f
    andwf freqDigits+6,f
    retlw 1 ; exit with w=1 to indicate value has changed

FreqZero    ; Set DDS frequency to 0 Hz
    ClearNregisters 8,freqDigits
FreqUpdate  ;Calc ctl word from digit array and update DDS frequency
    call MakeCtlWd
    SaveAccLo cw0 ;move the acc to the ctlwd storage register
    LoadAccLo cw0
    call SendCtlWd
    return

;The following routine was used for initial testing of communications between
;the PIC controller and the DDS chip. It loads a hard coded 1 MHz FTW to the 
;accumulator registers for transmission to the DDS chip.
;Not needed for production versions. So it may be commented out.
;But handy to have when nothing else seems to be working. :)
Set1MHz ;For 180MHz ref clock only.
    movlw 0x01
    movwf acc3
    movlw 0x6C
    movwf acc2
    movlw 0x16
    movwf acc1
    movlw 0xC1
    movwf acc0
    goto SendCtlWd
InitDDS 
    bcf DDSfq_ud
    bcf DDSclk
    return
SetDDSserial
;Need to toggle clk once to parallel load W0, thus setting serial mode
    bsf DDSclk
    bcf DDSclk
    bsf DDSfq_ud    ; toggle fq_ud to enable
    bcf DDSfq_ud
    return
SendCtlWd   ;Send Freq Ctrl Word to DDS chip
    movlw 0x01 ;DDS ctrl bits for AD9851
    btfsc ref125
    movlw 0x00 ;DDS ctrl bits for AD9850
    movwf acc4  ; sets DDS to 6x freq mult, power up, and zero phase
    bcf DDSfq_ud
    bcf DDSclk
    movlw 0x28	; send 40 bits (40=0x28)
    movwf count
LoopSCW
    bcf DDSdata	;default clear data bit
    rrf acc4,f  ;shift all 5 ctrl bytes right
    rrf acc3,f
    rrf acc2,f
    rrf acc1,f
    rrf acc0,f  ;next bit to be sent is in carry
    btfsc status,c  ;if 1, then set data out to 1
    bsf DDSdata
    bsf DDSclk	;toggle the clock to send the bit
    bcf DDSclk
    decfsz count,f
    goto LoopSCW    ;repeat until all bits sent
    bsf DDSfq_ud    ;trigger frequency update
    bcf DDSfq_ud
    return

MakeCtlWd   ;Generate DDS frequency control word
    LoadAccLo freqDigits
    SaveAccLo AdjDigits0 ;assume that any offset does not affect digits 0..3
    call clrAcc64 ;clear acc and R
    LoadAccHi freqDigits+4
; Check offset flag and if set, add offset value
    btfss plusOffset
    goto NoOffset ;no offset
    LoadRHi fqOffset4
    call AddDecimal
    
NoOffset
    SaveAccHi AdjDigits4
    call clrAcc64 ; Init the acc to zero
    movlw AdjDigits7	;load addr of frequency digits array
    movwf FSR	; and store in indirect ptr reg
    movlw 0x08  ; load count of digits
    movwf count
    goto LoopMCWenter ;skip the x10 for the 1st digit
LoopMCW
    call AccTimes10
LoopMCWenter
    movf indf,w
    call AddDigit
    decf fsr,f
    decfsz count,f
    goto LoopMCW
;
;All digits have been processed.

    btfsc acc3,7    ; Check fractional part to see if round up is req'd
    incfsz acc4,f   ;Yes, so increment integer part, and propagate carries.
    goto MCWshiftWord
    incfsz acc5,f
    goto MCWshiftWord
    incfsz acc6,f
    goto MCWshiftWord
    incf acc7,f
MCWshiftWord    ;move the result from acc4..acc7 to acc0..acc3
    LoadAccLo acc4
    return  ;Frequency/sweep Control Word is in acc0..acc3
AddDigit    ; Add next frequency coefficient to acc and mpy by 10
    ;coefficient is equal to digit value x fBase
    clrf r0 ;clear aux reg r
    clrf r1
    clrf r2
    clrf r3
    clrf r4
    clrf r5
    clrf r6
    clrf r7
    movwf temp ; save digit value
    movf temp,f ;test for zero
    btfsc status,z
    goto mpyDone
DigitMpyLoop ;repeated additions is simplest way to multiply by 0..9
    bcf status,c
    movf fBase0,w
    addwf r0,f
    btfsc status,c
    call carryR64_1
    movf fBase1,w
    addwf r1,f
    btfsc status,c
    call carryR64_2
    movf fBase2,w
    addwf r2,f
    btfsc status,c
    call carryR64_3
    movf fBase3,w
    addwf r3,f
    btfsc status,c
    call carryR64_4
    movf fBase4,w
    addwf r4,f
    btfsc status,c
    call carryR64_5
    movf fBase5,w
    addwf r5,f
    btfsc status,c 
    call carryR64_6
    decfsz temp,f
    goto DigitMpyLoop
    movlw 0 ;round up the result
    rlf r0,w ;set carry bit if req'd
    call carryr64_2
;shift result right one byte
    movf r1,w
    movwf r0
    movf r2,w
    movwf r1
    movf r3,w
    movwf r2
    movf r4,w
    movwf r3
    movf r5,w
    movwf r4
    movf r6,w
    movwf r5
    clrf r6
mpyDone
    call Add64
    return
carryR64_1   ;Chained skips to propagate carries in aux R reg
    incfsz r1,f
    return
carryr64_2
    incfsz r2,f
    return
carryr64_3
    incfsz r3,f
    return
carryr64_4
    incfsz r4,f
    return
carryr64_5
    incfsz r5,f
    return
carryr64_6
    incfsz r6,f
    return
    incf r7,f
    return

AccTimes10	;Multiply acc x 10
    ;copy acc to aux reg.
    call AccToR ;move acc contents to aux reg
    movlw 0x02
    call accLShiftN ;Shift left twice to get x4
    call Add64	; add the saved value back to the acc to get x5
    movlw 0x01
    call accLShiftN ;shift left once more to get x10
    return
AccLShiftN ;Shift acc left N times where N is in Wreg
    movwf count1
LoopALSN
    bcf status,c
    rlf acc0,f
    rlf acc1,f
    rlf acc2,f
    rlf acc3,f
    rlf acc4,f
    rlf acc5,f
    rlf acc6,f
    rlf acc7,f
    decfsz count1,f
    goto LoopALSN
    return
;
;  Utility subroutines
;
EEwrite ; Write to eeRam
    bank0
    bcf pir1,eeif
    bank1
;commented out following since interrupts not used
;    movf intcon,w
;    movwf saveIntcon  
    bsf eecon1,wren
    bcf intcon,gie    ; disable interrupts
    ;start of critical code
    movlw 0x55
    movwf eecon2
    movlw 0xAA
    movwf eecon2
    bsf eecon1,wr
    ;end of critical code
    bank0
    btfss pir1,eeif    ;wait for write to finish
    goto $-1
;commented out following since interrupts not used
;    movf saveIntcon,w
;    movwf Intcon    ;restore intcon register
    bank1
    movf eedata,w    ;compare data
    bsf eecon1,rd
    xorwf eedata,w
    return    ;if successful w=0 and z is set
getEEram
    bank1
    movwf eeadr
    bsf eecon1,rd
    movf eedata,w
    return
getNxtEEram
    bank1
    incf eeadr,f
    bsf eecon1,rd
    movf eedata,w
    return

    
;Misc 32/64 bit math routines
clrAcc64 ;clear 64 bit accumulator registers acc and r
    ClearNregisters d'16',acc0
    return


SaveAccHiInd ; Save high 4 bytes of acc to reg indirect
    movwf fsr ;start address is in w
    movf acc4,w
    movwf indf
    movf acc5,w
    incf fsr,f
    movwf indf
    movf acc6,w
    incf fsr,f
    movwf indf
    movf acc7,w
    incf fsr,f
    movwf indf
    return
SaveAccLoInd ; Save low 4 bytes of acc to reg indirect
    movwf fsr ;start address is in w
    movf acc0,w
    movwf indf
    movf acc1,w
    incf fsr,f
    movwf indf
    movf acc2,w
    incf fsr,f
    movwf indf
    movf acc3,w
    incf fsr,f
    movwf indf
    return
LoadAccHiInd ; Load high 4 bytes of acc from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf acc4
    incf fsr,f
    movf indf,w
    movwf acc5
    incf fsr,f
    movf indf,w
    movwf acc6
    incf fsr,f
    movf indf,w
    movwf acc7
    return
LoadAccLoInd ; Load low 4 bytes of acc from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf acc0
    incf fsr,f
    movf indf,w
    movwf acc1
    incf fsr,f
    movf indf,w
    movwf acc2
    incf fsr,f
    movf indf,w
    movwf acc3
    return
LoadRxLoInd ; Load low 4 bytes of aux reg r0 from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf r0
    incf fsr,f
    movf indf,w
    movwf r1
    incf fsr,f
    movf indf,w
    movwf r2
    incf fsr,f
    movf indf,w
    movwf r3
    return
LoadRxHiInd ; Load low 4 bytes of aux reg r0 from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf r4
    incf fsr,f
    movf indf,w
    movwf r5
    incf fsr,f
    movf indf,w
    movwf r6
    incf fsr,f
    movf indf,w
    movwf r7
    return
ClearNregistersInd
    ;Clear specified number of contiguous registers where number is in xfrCount
    ;and start register address is in w
    movwf fsr
LoopCNRI
    clrf indf
    incf fsr
    decfsz xfrCount,f
    goto LoopCNRI
    return
ShiftUpNregistersInd
    ;Shift bytes up across N registers
    ;highest register is in w
    movwf fsr
LoopSUNRI
    decf fsr,f
    movf indf,w
    incf fsr,f
    movwf indf
    decf fsr,f
    decfsz xfrCount,f
    goto LoopSUNRI
    return


AccToR ;move acc contents to aux reg.
    ;No indirects here, so that calling code can use FSR without it
    ;getting corrupted here.
    movf acc0,w	
    movwf R0
    movf acc1,w
    movwf R1
    movf acc2,w
    movwf R2
    movf acc3,w
    movwf R3
    movf acc4,w
    movwf R4
    movf acc5,w
    movwf R5
    movf acc6,w
    movwf R6
    movf acc7,w
    movwf R7
    return


add32 ;32 bit add routine
    movf R0,w
    addwf acc0,f
    btfsc status,c
    call carry1
    movf R1,w
    addwf acc1,f
    btfsc status,c
    call carry2
    movf R2,w
    addwf acc2,f
    btfsc status,c
    incf acc3,f
    movf R3,w
    addwf acc3,f
    return
neg32	;Negate the accumulator
    comf acc0,f
    comf acc1,f
    comf acc2,f
    comf acc3,f
    incfsz acc0,f ;increment and check for carry
    return ;no carry, so return
carry1
    incfsz acc1,f
    return
carry2
    incfsz acc2,f
    return
    incf acc3,f
    return
add64 ;64 bit add routine
    movf R0,w
    addwf acc0,f
    btfsc status,c
    call carry64_1
    movf R1,w
    addwf acc1,f
    btfsc status,c
    call carry64_2
    movf R2,w
    addwf acc2,f
    btfsc status,c
    call carry64_3
add40 ;Entry point for 40 bit add
    movf R3,w
    addwf acc3,f
    btfsc status,c
    call carry64_4
    movf R4,w
    addwf acc4,f
    btfsc status,c
    call carry64_5
    movf R5,w
    addwf acc5,f
    btfsc status,c
    call carry64_6
    movf R6,w
    addwf acc6,f
    btfsc status,c
    incf acc7,f
    movf R7,w
    addwf acc7,f
    return
carry64_1   ;Chained skips to propagate carries
    incfsz acc1,f
    return
carry64_2
    incfsz acc2,f
    return
carry64_3
    incfsz acc3,f
    return
carry64_4
    incfsz acc4,f
    return
carry64_5
    incfsz acc5,f
    return
carry64_6
    incfsz acc6,f
    return
    incf acc7,f
    return

NegDecimal ; Negate 8-digit unpacked decimal number in acc0..7
    movlw 0x08
    movwf count
    movlw acc0
    movwf fsr
LoopND
    movf indf,w
    sublw 0x09
    movwf indf
    incf fsr,f
    decfsz count
    goto LoopND
    ;Acc now contains nines complement. Now increment to get tens complement
    ClearNregisters 8,r0
    incf r0,f
    ;continue to AddDecimal to 
AddDecimal ; Eight digit unpacked BCD add; operands in acc0..7 and r0..7
    call Add64 ; add the two 8-digit numbers
;fsr indf
;Now adjust digits for carry
LoopAD2
    movlw 0xF6
    movwf r0
    movwf r1
    movwf r2
    movwf r3
    movwf r4
    movwf r5
    movwf r6
    movwf r7
;new code
    movlw acc0
    movwf fsr
    movlw 0x08
    movwf count
LoopAD1
    movf indf,w
    sublw 0x09
    rrf temp,f ; save carry bit
    incf fsr,f
    decfsz count,f
    goto LoopAD1
    comf temp,f ;
    btfsc status,z
    return ;no adjustment required
    movlw r0
    movwf fsr
    movlw 0x08
    movwf count
LoopAD3
    rrf temp,f
    btfss status,c
    clrf indf
    incf fsr
    decfsz count,f
    goto LoopAD3
    call Add64
    goto LoopAD2

; Delay routines
LongDelay
    movwf count
    movlw LongDelayMpy
LdelayLp
    call delay
    decfsz count,f
    goto LdelayLp
    return
delay40us   ;40 microsecond delay (approx), call is 2 cycles
    movlw Delay40usShort ;1 cycle
    movwf dlyctrL ;1 cycle
    decfsz dlyctrH,f ;1 cycle
    goto $-1 ;2 cycles
    return ;2 cycles

Delay1mS	 ; 00002 cycles for subroutine call
    movlw Delay1msLong      ; 00001 cycle - load long delay count
    call delay	 ; 770*w+4 cycles
    movlw Delay1msShort    ; 00001 cycle - load short delay count
    movwf dlyctrL ; 00001 cycle
    decfsz dlyctrL,f ; 3cycles/iteration
                 ; = 00216 total cycles (00744 for 4MHz)
    goto $-1	 ; -0001 cycle for loop exit
    nop		 ; 00001 cycle short tweak
    Return	 ; 00002 cycles, total 997 cycles

Delay10mS    ; 00002 cycles for subroutine call
    movlw Delay10msLong    ; 00001 cycle - long delay count
    call delay    ; 770*w+4 cycles
    movlw Delay10msShort    ; 00001 cycle - short delay count
    movwf dlyctrL    ; 00001 cycle
    decfsz dlyctrL,f    ; 3cycles/iteration
                        ; = 00462 total cycles (00744 for 4MHz)
    goto $-1    ; -0001 cycle for loop exit
    goto $+1    ; 00002 cycles - a short tweak
    goto $+1    ; +2 cycles reqd for 3.579 MHz xtal only
;   nop         ; +1 cycle reqd for 4 MHz xtal only
    Return    ; 00002 cycles, total 9997 cycles
delay ; Variable Delay subroutine: delay cycles = 770*w+4
    ; 00002 cyc for entry call
    movwf    dlyctrH    ; 00001 cyc, w contains delay value
subloop
    decfsz    dlyctrL,f    ; 00768 cyc for loop * w
    goto $-1    ; -0001 cyc for loop exit * w
    decfsz dlyctrH,f    ; 00001 * w
    goto subloop    ; 00002 * w
    ; -0001 for loop exit
    return ; 00002
;
	org 0x3ff
OscConst
	retlw	0x98
;Start of eeram
    org eebase
fOffsetSave
    de 0,1,6,0,0,0,0,0 ;optional offset is 1600 kHz (01600000 Hz)
ftw1
    ; RefOsc180MHz
      de 0x9C,0xE0,0xDE,0x65,0xDC,0x17 ;weight per digit coefficient 180Mhz
ftw2
    ; RefOsc125MHz
      de 0xAD,0x4D,0xD0,0x17,0x5C,0x22 ;weight per digit coefficient 125Mhz

;The following allows provision to modify the saved ftw coefficient, but have the 
;ability to restore the original setting if things go horribly wrong.

ftwFactorySetting1
    ; RefOsc180MHz
      de 0x9C,0xE0,0xDE,0x65,0xDC,0x17 ;weight per digit coefficient 180Mhz
ftwFactorySetting2
    ; RefOsc125MHz
      de 0xAD,0x4D,0xD0,0x17,0x5C,0x22 ;weight per digit coefficient 125Mhz
dummy_padding
      de 0x00,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA

    end